/**
 * JASMINe
 * Copyright (C) 2011 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: CorrelateCollectorService.java 8637 2011-07-25 09:48:02Z durieuxp $
 * --------------------------------------------------------------------------
 */

package org.ow2.jasmine.probe.collectors.correlate.internal;

import org.apache.felix.ipojo.annotations.*;
import org.ow2.jasmine.probe.JasmineIndicator;
import org.ow2.jasmine.probe.JasmineProbeException;
import org.ow2.jasmine.probe.JasminePropertyInfo;
import org.ow2.jasmine.probe.collector.JasmineCollector;
import org.ow2.jasmine.probe.collector.JasmineCollectorException;
import org.ow2.jasmine.probe.collector.JasmineCollectorService;
import org.ow2.jasmine.probe.collectors.ConstantCollector;
import org.ow2.jasmine.probe.collectors.JCollector;
import org.ow2.jasmine.probe.probemanager.ProbeManager;
import org.ow2.jasmine.probe.util.CsvConverter;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;

import java.util.*;

/**
 * @author durieuxp
 */
@Component(name="CorrelateCollectorService")
@Provides
public class CorrelateCollectorService implements JasmineCollectorService {
    /**
     * Logger.
     */
    protected Log logger = LogFactory.getLog(JCollector.class);

    /**
     * List of collectors
     */
    private Map<String, CorrelateCollector> collectorList = new HashMap<String, CorrelateCollector>();

    /**
     * Info about Properties
     */
    private List<JasminePropertyInfo> properties;

    /**
     * Property names
     */
    public final static String PROP_OP = "op";
    public final static String PROP_SOURCES = "sources";

    /**
     * List of supported operations
     */
    public final static int OP_ADD = 1;
    public final static int OP_SUB = 2;
    public final static int OP_MUL = 3;
    public final static int OP_DIV = 4;
    public final static int OP_PERCENT = 5;

    // --------------------------------------------------------------------------------
    // ipojo management
    // --------------------------------------------------------------------------------

    @ServiceProperty(name="indicator.type", value="correlate")
    private String indicatorType;

    @Validate
    public void start() {
        // init the list of properties
        properties = new ArrayList<JasminePropertyInfo>();
        properties.add(new JasminePropertyInfo(PROP_OP, "correlate operation (add|sub|mul|div|percent)", true));
        properties.add(new JasminePropertyInfo(PROP_SOURCES, "csv list of source indicator name oor constants", true));
    }

    @Invalidate
    public void stop() {
    }

    @Requires
    ProbeManager probeManager = null;

    // --------------------------------------------------------------------------------
    //  JasmineCollectorService implementation
    // --------------------------------------------------------------------------------

    /**
     * Get a Collector for this indicator and this probe
     * @param indicator
     * @param period
     * @param probeid
     * @return the JasmineCollector
     * @throws JasmineCollectorException
     */
    public JasmineCollector getCollector(JasmineIndicator indicator, int period, String probeid)
            throws JasmineCollectorException {

        // Build a unique key from the probeid and the indicator name
        String key = probeid + ":" + indicator.getName();

        CorrelateCollector collector = collectorList.get(key);
        if (collector == null) {
            // sanity check, not really needed.
            if (! indicator.getType().equals(indicatorType)) {
                logger.error("Bad type: " + indicator.getType());
                throw new JasmineCollectorException("Bad type: " + indicator.getType());
            }

            Map<String, String> props = indicator.getProperties();

            // Get the collectors corresponding to the source indicators
            Collection<JasmineCollector> sources = new ArrayList<JasmineCollector>();
            Collection<String> inames = CsvConverter.csv2list(props.get(PROP_SOURCES));
            if (inames == null) {
                String err = "No sources defined in " + key;
                logger.error(err);
                throw new JasmineCollectorException(err);
            }
            for (String iname : inames) {
                JasmineCollector source = null;
                if (isNumeric(iname)) {
                    source = new ConstantCollector(iname, indicator, period);
                } else {
                    try {
                        source = probeManager.getCollector(iname, period, probeid);
                    } catch (JasmineProbeException e) {
                        logger.error("Cannot get value: " + e);
                        throw new JasmineCollectorException(e.getMessage());
                    }
                }
                sources.add(source);
            }

            // Check the operation
            String operation = props.get(PROP_OP);
            if (operation == null) {
                String err = "Operation not defined in " + key;
                logger.error(err);
                throw new JasmineCollectorException(err);
            }
            int ope = 0;
            if (operation.equalsIgnoreCase("add")) {
                ope = OP_ADD;
            } else if (operation.equalsIgnoreCase("sub")) {
                ope = OP_SUB;
            } else if (operation.equalsIgnoreCase("mul")) {
                ope = OP_MUL;
            }  else if (operation.equalsIgnoreCase("div")) {
                ope = OP_DIV;
            }  else if (operation.equalsIgnoreCase("percent")) {
                ope = OP_PERCENT;
            } else {
                String err = "Operation not supported in CorrelateCollectorService: " + operation;
                logger.error(err);
                throw new JasmineCollectorException(err);
            }

            // Create the Collector
            collector = new CorrelateCollector(key, indicator, period, ope, sources);
            collectorList.put(key, collector);
        }
        return collector;
    }


    /**
     * Retrieve the description of all specific properties hidden in JasmineOutput
     * Return a Map of all properties with their description
     * @return  a Map of property descriptions
     */
    public List<JasminePropertyInfo> getPropertiesInfo() {
        return properties;
    }

    // ----------------------------------------------------------------------------------------
    // private methods
    // ----------------------------------------------------------------------------------------

    private boolean isNumeric(String s) {
        return s.matches("((-|\\+)?[0-9]+(\\.[0-9]+)?)+");
    }
}
